from cfg_checker.common import logger_cli
from cfg_checker.common.settings import ENV_TYPE_SALT, ENV_TYPE_KUBE
from cfg_checker.common.exception import CheckerException
from cfg_checker.helpers import args_utils
from cfg_checker.modules.network import checker, mapper, pinger


command_help = "Network infrastructure checks and reports"
supported_envs = [ENV_TYPE_SALT, ENV_TYPE_KUBE]


def _selectClass(_env, strClassHint="checker"):
    _class = None
    if _env == ENV_TYPE_SALT:
        if strClassHint == "checker":
            _class = checker.SaltNetworkChecker
        elif strClassHint == "mapper":
            _class = mapper.SaltNetworkMapper
    elif _env == ENV_TYPE_KUBE:
        if strClassHint == "checker":
            _class = checker.KubeNetworkChecker
        elif strClassHint == "mapper":
            _class = mapper.KubeNetworkMapper
    if not _class:
        raise CheckerException(
            "Unknown hint for selecting Network handler Class: '{}'".format(
                strClassHint
            )
        )
    else:
        return _class


def init_parser(_parser):
    # network subparser
    net_subparsers = _parser.add_subparsers(dest='type')

    net_check_parser = net_subparsers.add_parser(
        'check',
        help="Do network check and print the result"
    )

    net_check_parser.add_argument(
        '--detailed',
        action="store_true", default=False,
        help="Print error details after summary"
    )

    net_check_parser.add_argument(
        '--skip-ifs',
        metavar='skip_ifs', default="docker",
        help="String with keywords to skip networks which has interfaces "
        "names with keywords as substrings. Example: 'eno' keyword will "
        "cause to skip interface named 'eno1np0'. Example: 'docker'"
    )

    net_report_parser = net_subparsers.add_parser(
        'report',
        help="Generate network check report"
    )

    net_report_parser.add_argument(
        '--html',
        metavar='network_html_filename',
        help="HTML filename to save report"
    )

    net_ping_parser = net_subparsers.add_parser(
        'ping',
        help="Ping all nodes with each other using network CIDR"
    )

    net_ping_parser.add_argument(
        '--cidr',
        metavar='network_ping_cidr',
        help="Subnet CIDR to ping nodes in"
    )
    net_ping_parser.add_argument(
        '--mtu',
        metavar='network_ping_mtu',
        help="MTU size to use. Ping will be done for MTU - 20 - 8"
    )
    net_ping_parser.add_argument(
        '--detailed',
        action="store_true", default=False,
        help="Print detailed report at the end"
    )

    net_subparsers.add_parser(
        'map',
        help="Print network map"
    )

    net_subparsers.add_parser(
        'list',
        help="List networks in reclass"
    )

    return _parser


def do_check(args, config):
    # Net Checks
    # should not print map, etc...
    # Just bare summary and errors
    # Check if there is supported env found
    _env = args_utils.check_supported_env(
        [ENV_TYPE_SALT, ENV_TYPE_KUBE],
        args,
        config
    )
    # prepare skip keywords
    _skip_ifs_keywords = []
    for _str in args.skip_ifs.split(","):
        _skip_ifs_keywords.append(_str)
    logger_cli.info(
        "-> Interface keywords skip list is '{}'".format(
            ", ".join(_skip_ifs_keywords)
        )
    )
    # Start command
    logger_cli.info("# Network check to console")
    _skip, _skip_file = args_utils.get_skip_args(args)
    _class = _selectClass(_env)
    netChecker = _class(
        config,
        skip_list=_skip,
        skip_list_file=_skip_file
    )
    netChecker.check_networks(skip_keywords=_skip_ifs_keywords)

    # save what was collected
    netChecker.errors.save_iteration_data()

    # print a report
    netChecker.print_summary()

    # if set, print details
    if args.detailed:
        netChecker.print_error_details()


def do_report(args, config):
    # Network Report
    # Check if there is supported env found
    _env = args_utils.check_supported_env(
        [ENV_TYPE_SALT],
        args,
        config
    )
    # Start command
    logger_cli.info("# Network report (check, node map")

    _filename = args_utils.get_arg(args, 'html')
    _skip, _skip_file = args_utils.get_skip_args(args)
    _class = _selectClass(_env)
    netChecker = _class(
        config,
        skip_list=_skip,
        skip_list_file=_skip_file
    )
    netChecker.check_networks(map=False)

    # save what was collected
    netChecker.errors.save_iteration_data()
    netChecker.create_html_report(_filename)

    return


def do_map(args, config):
    # Network Map
    # Check if there is supported env found
    _env = args_utils.check_supported_env(
        [ENV_TYPE_SALT, ENV_TYPE_KUBE],
        args,
        config
    )
    # Start command
    logger_cli.info("# Network report")
    _skip, _skip_file = args_utils.get_skip_args(args)
    _class = _selectClass(_env, strClassHint="mapper")
    networkMap = _class(
        config,
        skip_list=_skip,
        skip_list_file=_skip_file
    )
    networkMap.map_networks()
    networkMap.create_map()
    networkMap.print_map()

    return


def do_list(args, config):
    # Network List
    # Check if there is supported env found
    _env = args_utils.check_supported_env(
        [ENV_TYPE_SALT, ENV_TYPE_KUBE],
        args,
        config
    )
    # Start command
    _skip, _skip_file = args_utils.get_skip_args(args)
    _class = _selectClass(_env, strClassHint="mapper")
    _map = _class(
        config,
        skip_list=_skip,
        skip_list_file=_skip_file
    )
    logger_cli.info("# Mapping networks")
    if _env == ENV_TYPE_SALT:
        reclass = _map.map_network(_map.RECLASS)
        _s = [str(_n) for _n in reclass.keys()]
        logger_cli.info("\n# Reclass networks list")
        logger_cli.info("\n".join(_s))

    runtime = _map.map_network(_map.RUNTIME)
    logger_cli.info("\n# Runtime networks list")
    for _net, _nodes in runtime.items():
        _names = set()
        _mtus = set()
        for _node_name, _interfaces in _nodes.items():
            for _if in _interfaces:
                _names.add(_if["name"])
                _mtus.add(_if["mtu"])
        logger_cli.info(
            "{:21}: {}, {}".format(
                str(_net),
                "/".join(list(_names)),
                "/".join(list(_mtus))
            )
        )
    # Done
    return


def do_ping(args, config):
    # Network pinger
    # Checks if selected nodes are pingable
    # with a desireble parameters: MTU, Frame, etc
    # Check if there is supported env found
    _env = args_utils.check_supported_env(ENV_TYPE_SALT, args, config)
    # Start command
    if not args.cidr:
        logger_cli.error("\n# Use mcp-check network list to get list of CIDRs")
    _cidr = args_utils.get_arg(args, "cidr")
    _skip, _skip_file = args_utils.get_skip_args(args)
    # init mapper
    _skip, _skip_file = args_utils.get_skip_args(args)
    _class = _selectClass(_env, strClassHint="mapper")
    _mapper = _class(
        config,
        skip_list=_skip,
        skip_list_file=_skip_file
    )
    # init pinger
    _pinger = pinger.NetworkPinger(
        _mapper,
        mtu=args.mtu,
        detailed=args.detailed,
        skip_list=_skip,
        skip_list_file=_skip_file
    )

    _ret = _pinger.ping_nodes(_cidr)

    if _ret < 0:
        # no need to save the iterations and summary
        return
    else:
        # save what was collected
        _pinger.mapper.errors.save_iteration_data()

        # print a report
        _pinger.print_summary()

        # if set, print details
        if args.detailed:
            _pinger.print_details()

        return


def do_trace(args, config):
    # Network packet tracer
    # Check if packet is delivered to proper network host
    logger_cli.info("# Packet Tracer not yet implemented")

    # TODO: Packet tracer

    return
